/*
	񕄍EtHgZbg쐬
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "fontmisaki.h"
#include "fontk6x8.h"
#include "font4x8.h"

#define FALSE	0
#define TRUE	1

/*
	SJISJISɕϊ (getJis̉)
*/
static unsigned char *sjis2jis(unsigned char *jis, const unsigned char *sjis)
{
	jis[0] = sjis[0];
	jis[1] = sjis[1];

	if(!(jis[0] & 0x80))
		return jis;

	if(jis[0] >= 0xe0)
		jis[0] -= 0x40;
	if(jis[1] >= 0x80)
		jis[1]--;
	if(jis[1] >= 0x9e) {
		jis[0] = (jis[0] - 0x70) << 1;
		jis[1] -= 0x7d;
	} else {
		jis[0] = ((jis[0] - 0x70) << 1) - 1;
		jis[1] -= 0x1f;
	}
	return jis;
}

/*
	JISR[h𓾂 (encodeText̉)
*/
static unsigned short getJis(const unsigned char **text)
{
	const unsigned short ascii2jis[] = {
		0x0000, /* 0x00 NUL */
		0x2121, /* 0x01 SOH */
		0x2121, /* 0x02 STX */
		0x2121, /* 0x03 ETX */
		0x2121, /* 0x04 EOT */
		0x2121, /* 0x05 ENQ */
		0x2121, /* 0x06 ACK */
		0x2121, /* 0x07 BEL */

		0x2121, /* 0x08 BS */
		0x2121, /* 0x09 TAB */
		0x0000, /* 0x0a LF */
		0x2121, /* 0x0b VT */
		0x2121, /* 0x0c FF */
		0x0000, /* 0x0d CR */
		0x2121, /* 0x0e SO */
		0x2121, /* 0x0f SI */

		0x2121, /* 0x10 DLE */
		0x2121, /* 0x11 DC1 */
		0x2121, /* 0x12 DC2 */
		0x2121, /* 0x13 DC3 */
		0x2121, /* 0x14 DC4 */
		0x2121, /* 0x15 NAK */
		0x2121, /* 0x16 SYN */
		0x2121, /* 0x17 ETB */

		0x2121, /* 0x18 CAN */
		0x2121, /* 0x19 EM */
		0x2121, /* 0x1a SUB */
		0x2121, /* 0x1b ESC */
		0x2121, /* 0x1c FS */
		0x2121, /* 0x1d GS */
		0x2121, /* 0x1e RS */
		0x2121, /* 0x1f US */

		0x2121, /* SPACE */
		0x212a, /* ! */
		0x2149, /* " */
		0x2174, /* # */
		0x2170, /* $ */
		0x2173, /* % */
		0x2175, /* & */
		0x2147, /* ' */

		0x214a, /* ( */
		0x214b, /* ) */
		0x2176, /* * */
		0x215c, /* + */
		0x2124, /* , */
		0x215d, /* - */
		0x2125, /* . */
		0x213f, /* / */

		0x2330, /* 0 */
		0x2331, /* 1 */
		0x2332, /* 2 */
		0x2333, /* 3 */
		0x2334, /* 4 */
		0x2335, /* 5 */
		0x2336, /* 6 */
		0x2337, /* 7 */

		0x2338, /* 8 */
		0x2339, /* 9 */
		0x2127, /* : */
		0x2128, /* ; */
		0x2163, /* < */
		0x2161, /* = */
		0x2164, /* > */
		0x2129, /* ? */

		0x2177, /* @ */
		0x2341, /* A */
		0x2342, /* B */
		0x2343, /* C */
		0x2344, /* D */
		0x2345, /* E */
		0x2346, /* F */
		0x2347, /* G */

		0x2348, /* H */
		0x2349, /* I */
		0x234a, /* J */
		0x234b, /* K */
		0x234c, /* L */
		0x234d, /* M */
		0x234e, /* N */
		0x234f, /* O */

		0x2350, /* P */
		0x2351, /* Q */
		0x2352, /* R */
		0x2353, /* S */
		0x2354, /* T */
		0x2355, /* U */
		0x2356, /* V */
		0x2357, /* W */

		0x2358, /* X */
		0x2359, /* Y */
		0x235a, /* Z */
		0x214e, /* [ */
		0x216f, /* \ */
		0x214f, /* ] */
		0x2130, /* ^ */
		0x2132, /* _ */

		0x212e, /* ` */
		0x2361, /* a */
		0x2362, /* b */
		0x2363, /* c */
		0x2364, /* d */
		0x2365, /* e */
		0x2366, /* f */
		0x2367, /* g */

		0x2368, /* h */
		0x2369, /* i */
		0x236a, /* j */
		0x236b, /* k */
		0x236c, /* l */
		0x236d, /* m */
		0x236e, /* n */
		0x236f, /* o */

		0x2370, /* p */
		0x2371, /* q */
		0x2372, /* r */
		0x2373, /* s */
		0x2374, /* t */
		0x2375, /* u */
		0x2376, /* v */
		0x2377, /* w */

		0x2378, /* x */
		0x2379, /* y */
		0x237a, /* z */
		0x2150, /* { */
		0x2143, /* | */
		0x2151, /* } */
		0x2141, /* ~ */
		0x2121, /* DEL */

		0xffff, /* 0x80 */
		0xffff, /* 0x81 */
		0xffff, /* 0x82 */
		0xffff, /* 0x83 */
		0xffff, /* 0x84 */
		0xffff, /* 0x85 */
		0xffff, /* 0x86 */
		0xffff, /* 0x87 */

		0xffff, /* 0x88 */
		0xffff, /* 0x89 */
		0xffff, /* 0x8a */
		0xffff, /* 0x8b */
		0xffff, /* 0x8c */
		0xffff, /* 0x8d */
		0xffff, /* 0x8e */
		0xffff, /* 0x8f */

		0xffff, /* 0x90 */
		0xffff, /* 0x91 */
		0xffff, /* 0x92 */
		0xffff, /* 0x93 */
		0xffff, /* 0x94 */
		0xffff, /* 0x95 */
		0xffff, /* 0x96 */
		0xffff, /* 0x97 */

		0xffff, /* 0x98 */
		0xffff, /* 0x99 */
		0xffff, /* 0x9a */
		0xffff, /* 0x9b */
		0xffff, /* 0x9c */
		0xffff, /* 0x9d */
		0xffff, /* 0x9e */
		0xffff, /* 0x9f */

		0xffff, /* 0xa0 */
		0x2123, /* B */
		0x2156, /* u */
		0x2157, /* v */
		0x2122, /* A */
		0x2126, /* E */
		0x2572, /*  */
		0x2521, /* @ */

		0x2523, /* B */
		0x2525, /* D */
		0x2527, /* F */
		0x2529, /* H */
		0x2563, /*  */
		0x2565, /*  */
		0x2567, /*  */
		0x2543, /* b */

		0x213c, /* [ */
		0x2522, /* A */
		0x2524, /* C */
		0x2526, /* E */
		0x2528, /* G */
		0x252a, /* I */
		0x252b, /* J */
		0x252d, /* L */

		0x252f, /* N */
		0x2531, /* P */
		0x2533, /* R */
		0x2535, /* T */
		0x2537, /* V */
		0x2539, /* X */
		0x253b, /* Z */
		0x253d, /* \ */

		0x253f, /* ^ */
		0x2541, /* ` */
		0x2544, /* c */
		0x2546, /* e */
		0x2548, /* g */
		0x254a, /* i */
		0x254b, /* j */
		0x254c, /* k */

		0x254d, /* l */
		0x254e, /* m */
		0x254f, /* n */
		0x2552, /* q */
		0x2555, /* t */
		0x2558, /* w */
		0x255b, /* z */
		0x255e, /* } */

		0x255f, /* ~ */
		0x2560, /*  */
		0x2561, /*  */
		0x2562, /*  */
		0x2564, /*  */
		0x2566, /*  */
		0x2568, /*  */
		0x2569, /*  */

		0x256a, /*  */
		0x256b, /*  */
		0x256c, /*  */
		0x256d, /*  */
		0x256f, /*  */
		0x2573, /*  */
		0x212b, /* J */
		0x212c, /* K */

		0xffff, /* 0xe0 */
		0xffff, /* 0xe1 */
		0xffff, /* 0xe2 */
		0xffff, /* 0xe3 */
		0xffff, /* 0xe4 */
		0xffff, /* 0xe5 */
		0xffff, /* 0xe6 */
		0xffff, /* 0xe7 */

		0xffff, /* 0xe8 */
		0xffff, /* 0xe9 */
		0xffff, /* 0xea */
		0xffff, /* 0xeb */
		0xffff, /* 0xec */
		0xffff, /* 0xed */
		0xffff, /* 0xee */
		0xffff, /* 0xef */

		0xffff, /* 0xf0 */
		0xffff, /* 0xf1 */
		0xffff, /* 0xf2 */
		0xffff, /* 0xf3 */
		0xffff, /* 0xf4 */
		0xffff, /* 0xf5 */
		0xffff, /* 0xf6 */
		0xffff, /* 0xf7 */

		0xffff, /* 0xf8 */
		0xffff, /* 0xf9 */
		0xffff, /* 0xfa */
		0xffff, /* 0xfb */
		0xffff, /* 0xfc */
		0xffff, /* 0xfd */
		0xffff, /* 0xfe */
		0xffff  /* 0xff */
	};
	unsigned short jis;
	unsigned char buf[2];

	jis = ascii2jis[**text];
	if (jis == 0x0000) {
		(*text)++;
		return jis;
	} else if(jis == 0xffff) {
		sjis2jis(buf, *text);
		(*text) += 2;
		return buf[0] << 8U | buf[1];
	} else {
		(*text) += 1;
		return jis;
	}
}

/*
	R[h\瓾. \ɂȂΓo^ (encodeText̉)
*/
static int getCode(unsigned short *code_table, unsigned short jis)
{
	unsigned short *p;

	for(p = code_table; p < code_table + 0x100; p++)
		if(*p == jis)
			return (int )(p - code_table);

	for(p = code_table; p < code_table + 0x100; p++)
		if(*p == 0) {
			*p = jis;
			return (int )(p - code_table);
		}

	return -1;
}

/*
	𕄍, R[h\ɓo^
*/
int encodeText(unsigned char *code, unsigned short *code_table, const unsigned char *text)
{
	int c;
	unsigned short jis;
	unsigned char *p = code;

	while((jis = getJis(&text)) > 0) {
		if((c = getCode(code_table, jis)) < 0)
			return -1;
		*p++ = (unsigned char )c;
	}

	return (int )(p - code);
}

/*
	7bitl (writeFont̉)
*/
static void write7(unsigned char *buf, int address, int value)
{
	int i;
	unsigned char mask = (1 << (address % 8));

	for(i = 0; i < 7; i++)
		if(value & (0x80 >> (i + 1)))
			buf[(address / 8) * 7 + i] |= mask;
		else
			buf[(address / 8) * 7 + i] &= ~mask;
}

/*
	tHg (makeFont̉)
*/
static void writeFont(unsigned char *buf, int *address, int jis)
{
	int i, index = ((jis >> 8) - 0x21) * 94 + ((jis & 0xff) - 0x21);

	for(i = 0; i < 7; i++) {
		write7(buf, *address, fontMisaki[index][i]);
		(*address)++;
	}
}

/*
	tHgZbg쐬
*/
int makeFont(unsigned char *buf, const unsigned short *code_table)
{
	int i, address = 0;

	for(i = 0; i < 256 && code_table[i] != 0; i++)
		writeFont(buf, &address, code_table[i]);

	return ((address + 7) / 8) * 7;
}

/* eof */
